/*
 * Copyright (c) 2022 The BytePlusRTC project authors. All Rights Reserved.
 * @brief RTS
*/

#import <UIKit/UIKit.h>
#import "rtc/ByteRTCRtsDefines.h"

@class RTS;
@class RTSRoom;

#pragma mark - RTSDelegate
/** 
 * @type callback
 * @brief RTSDelegate protocol contains the callback method provided by the SDK, which reports some run-time events to the application through a proxy
 */
@protocol RTSDelegate <NSObject>
@optional

#pragma mark - Engine Management Delegate Methods
/** 
 * @type callback
 * @region engine management
 * @brief Callback about the status change of the SDK's network connection to the signaling server.
 * @param engine RTS object
 * @param state The current connection status between the SDK and the signaling server. See ByteRTCConnectionState{@link #ByteRTCConnectionState}.
 * @notes Refer to [Getting Connection Status](https:
 */
- (void)rtsEngine:(RTS * _Nonnull)engine onConnectionStateChanged:(ByteRTCConnectionState)state;
/** 
 * @type callback
 * @region Engine management
 * @brief Callback for the current network connection type change.
 * @param engine RTS object.
 * @param type The current network connection type. See ByteRTCNetworkType{@link #ByteRTCNetworkType}.
 */
- (void)rtsEngine:(RTS * _Nonnull)engine onNetworkTypeChanged:(ByteRTCNetworkType)type;

/** 
 * @type callback
 * @region Engine Management
 * @brief Callback for failed room creation.
 * @param engine RTS object.
 * @param roomId Room ID.
 * @param errorCode Create room error code: <br>
 *        + -1007: Invalid roomId. <br>
 *        + -1013: A room with the same roomId already exists.
 */
- (void)rtsEngine:(RTS *_Nonnull)engine onCreateRoomStateChanged:(NSString * _Nonnull)roomId errorCode:(NSInteger)errorCode;


/** 
 * @type callback
 * @region proxy
 * @brief Callback on local proxy connection. After calling setLocalProxy:{@link #RTS#setLocalProxy:} to set local proxies, you will receive this callback that informs you of the states of local proxy connection. 
 * @param type The types of local proxies. Refer to ByteRTCLocalProxyType{@link #ByteRTCLocalProxyType} for details.  <br>
 * @param state The states of local proxy connection. Refer to ByteRTCLocalProxyState{@link #ByteRTCLocalProxyState} for details.  <br>
 * @param error The errors of local proxy connection. Refer to ByteRTCLocalProxyError{@link #ByteRTCLocalProxyError} for details.
 */
- (void)rtsEngine:(RTS * _Nonnull)engine onLocalProxyStateChanged:(ByteRTCLocalProxyType)type withProxyState:(ByteRTCLocalProxyState)state 
        withProxyError:(ByteRTCLocalProxyError)error;

#pragma mark - Data Statistics Delegate Methods
/** 
 * @type callback
 * @region engine management
 * @brief Callback triggered every 2 seconds about the usage of CPU and memory.
 * @param engine RTS object
 * @param stats Usage of CPU and memory. See ByteRTCSysStats{@link #ByteRTCSysStats}.
 */
- (void)rtsEngine:(RTS * _Nonnull)engine onSysStats:(const ByteRTCSysStats * _Nonnull)stats;

#pragma mark - Real-time Messaging Delegate Methods
/** 
 * @type callback
 * @region Real-time messaging
 * @brief Login result callback
 * @param engine RTS object
 * @param uid Login user ID
 * @param errorCode Login result. See ByteRTCLoginErrorCode{@link #ByteRTCLoginErrorCode}.
 * @param elapsed The time in ms it takes from calling login{@link #RTS#login} to the result returned.
 * @notes This callback is received after login{@link #RTS#login} is called.
 */
- (void)rtsEngine:(RTS * _Nonnull)engine onLoginResult:(NSString * _Nonnull)uid
        errorCode:(ByteRTCLoginErrorCode)errorCode elapsed:(NSInteger)elapsed;
/** 
 * @type callback
 * @region Real-time messaging
 * @brief Callback of logout result
 * @param engine RTS object
 * @param reason It describes the reason why users log out. See ByteRTCLogoutReason{@link #ByteRTCLogoutReason} 
 * @notes You will receive this callback when calling logout{@link #RTS#logout} or when the local user is kicked out because another user logs in with the same UserId.
 */
- (void)rtsEngine:(RTS * _Nonnull)engine onLogout:(ByteRTCLogoutReason)reason;
/** 
 * @type callback
 * @region Real-time messaging
 * @brief Callback for the result of setting application server parameters.
 * @param engine RTS object
 * @param errorCode  <br>
 *         + 200, set successfully <br>
 *         + Return other, set failed
 * @notes This callback is received after setServerParams:url:{@link #RTS#setServerParams:url:} is called.
 */
- (void)rtsEngine:(RTS * _Nonnull)engine onServerParamsSetResult:(NSInteger)errorCode;
/** 
 * @type callback
 * @region Real-time messaging
 * @brief Callback for the result of querying the login status of the remote or local user.
 * @param engine RTS object.
 * @param peerUserId ID of the user to be queried.
 * @param status The login status of the user queried. See ByteRTCUserOnlineStatus{@link #ByteRTCUserOnlineStatus}.
 * @notes You must first call getPeerOnlineStatus:{@link #RTS#getPeerOnlineStatus:} to receive this callback.
 */
- (void)rtsEngine:(RTS * _Nonnull)engine onGetPeerOnlineStatus:(NSString * _Nonnull)peerUserId
           status:(ByteRTCUserOnlineStatus)status;
/** 
 * @type callback
 * @region Real-time messaging
 * @brief The local user receives the text message from a remote user calling sendMessage:message:config:{@link #RTS#sendMessage:message:config:} through this callback.
 * @param engine RTS object
 * @param uid User ID of the message sender.
 * @param message Received text message content.
 */
- (void)rtsEngine:(RTS * _Nonnull)engine onMessageReceived:(NSString * _Nonnull)uid
          message:(NSString * _Nonnull)message;
/** 
 * @type callback
 * @region Real-time messaging
 * @brief The local user receives the binary message from a remote user calling sendBinaryMessage:message:config:{@link #RTS#sendBinaryMessage:message:config:} through this callback.
 * @param engine RTS object.
 * @param uid User ID of the message sender.
 * @param message Received binary message content.
 */
- (void)rtsEngine:(RTS * _Nonnull)engine onBinaryMessageReceived:(NSString * _Nonnull)uid
          message:(NSData * _Nonnull)message;
/** 
 * @type callback
 * @region Real-time messaging
 * @brief Callback about the sending result of a p2p message.
 * @param engine RTS object
 * @param msgid The ID of this message. All P2P and P2Server messages share a single ID sequence.
 * @param error Message sending result, see ByteRTCUserMessageSendResult{@link #ByteRTCUserMessageSendResult}.
 * @notes The local user will receive this callback when a message is sent by calling sendMessage:message:config:{@link #RTS#sendMessage:message:config:} or sendBinaryMessage:message:config:{@link #RTS#sendBinaryMessage:message:config:}.
 */
- (void)rtsEngine:(RTS * _Nonnull)engine onMessageSendResult:(int64_t)msgid
            error:(ByteRTCUserMessageSendResult)error;
/** 
 * @type callback
 * @region Real-time messaging
 * @brief Callback about the sending result of a p2server message.
 * @param engine RTS object
 * @param msgid The ID of this message. All P2P and P2Server messages share a single ID sequence.
 * @param error Message sending result, see ByteRTCUserMessageSendResult{@link #ByteRTCUserMessageSendResult}.
 * @notes The local user will receive this callback when a message is sent by calling sendServerMessage:{@link #RTS#sendServerMessage:} or sendServerBinaryMessage:{@link #RTS#sendServerBinaryMessage:}.
 */
- (void)rtsEngine:(RTS * _Nonnull)engine onServerMessageSendResult:(int64_t)msgid
            error:(ByteRTCUserMessageSendResult)error message:(NSData * _Nonnull)message;

@end

#pragma mark - RTS

/**
 * 
 * @type api
 */
BYTERTC_APPLE_EXPORT @interface RTS : NSObject

/** 
 * @type callback
 */
@property(nonatomic, weak) id<RTSDelegate> _Nullable delegate;

#pragma mark - Engine Management Methods
/** 
 * @type api
 * @region Engine Management
 * @brief Creates an engine instance.   <br>
 *        This is the very first API that you must call if you want to use all the RTS capabilities.  <br>
 *        If there is no engine instance in current thread, calling this API will create one. If an engine instance has been created, calling this API again will create another instance.
 * @param appId A unique identifier for each App, randomly generated by the RTC console. Engine instances created with different AppIds cannot communicate with each other.
 * @param delegate Delegate sent from SDK to App. See RTSDelegate{@link #RTSDelegate}.
 * @param parameters Reserved parameter, in JSON string.
 * @return A RTSDelegate{@link #RTSDelegate} instance.
 */
+ (RTS * _Nullable)createRTS:(NSString * _Nonnull)appId
                    delegate:(id<RTSDelegate> _Nullable)delegate
                  parameters:(NSDictionary* _Nullable)parameters;
/** 
 * @type api
 * @region Engine Management
 * @brief Destroys the engine instance.
 * @notes  <br>
 *       + Call this API after all business scenarios related to the engine instance are destroyed.  <br>
 *       + When the API is called, RTC SDK destroys all memory associated with the engine instance and stops any interaction with the media server.  <br>
 *       + Calling this API will start the SDK exit logic. The engine thread is held until the exit logic is complete. The engine thread is retained until the exit logic is complete. Therefore, do not call this API directly in the callback thread, or it will cause a deadlock. This function takes a long time to execute, so it's not recommended to call this API in the main thread, or the main thread may be blocked.
 *       + You can enable ARC for Objective-C, to automatically trigger the destruction logic when the dealloc method is called.
 */
+ (void)destroyRTS;

/** 
 * @type api
 * @region  engine management
 * @brief Gets the current version number of the SDK.
 * @return The current version number.
 */
+ (NSString * _Nonnull)getSdkVersion;

/** 
 * @type api
 * @region Engine Management
 * @brief Configures the local log parameters of RTC SDK, including the logging level, directory, and the limits for total log file size.
 * @param logConfig Local log parameters. See ByteRTCLogConfig{@link #ByteRTCLogConfig}.
 * @return <br>
 *        + 0: Success.
 *        + –1: Failure. This API must be called before creating engine.
 *        + –2: Failure. Invalid parameters.
 * @notes This API must be called before createRTS:delegate:parameters:{@link #RTS#createRTS:delegate:parameters:}.
 */
+ (int) setLogConfig:(ByteRTCLogConfig *_Nonnull) logConfig;

/** 
 * @type api
 * @region Engine management
 * @brief Sets the business ID.   <br>
 *        You can use businessId to distinguish different business scenarios. You can customize your businessId to serve as a sub AppId, which can share and refine the function of the AppId, but it does not need authentication.
 * @param businessId Your customized businessId. BusinessId is a tag, and you can customize its granularity.
 * @return  <br>
 *         + 0: Success <br>
 *         + < 0: Failure <br>
 *         + -6001: The user is already in the room. <br>
 *         + -6002: The input is invalid. Legal characters include all lowercase letters, uppercase letters, numbers, and four other symbols, including '.', '-','_', and '@'.
 * @notes You must call this API before the joinRoom{@link #RTSRoom#joinRoom} API, otherwise it will be invalid.
 */
- (int)setBusinessId:(NSString* _Nullable)businessId;
/** 
 * @type api
 * @brief Sets runtime parameters/
 * @param parameters Reserved parameters.
 * @return  <br>
 *        + 0: Success.
 *        + < 0 : Fail. See ByteRTCReturnStatus{@link #ByteRTCReturnStatus} for more details
 */
- (int)setRuntimeParameters:(NSDictionary * _Nullable)parameters;

#pragma mark - Multi-room Methods
/** 
 * @type api
 * @region Room Management
 * @brief Create a room instance.
 *        This API only returns a room instance. You still need to call joinRoom{@link #RTSRoom#joinRoom} to actually create/join the room.
 *        Each call of this API creates one RTSRoom{@link #RTSRoom} instance. Call this API as many times as the number of rooms you need, and then call joinRoom{@link #RTSRoom#joinRoom} of each RTSRoom instance to join multiple rooms at the same time.
 *        In multi-room mode, a user can subscribe to media streams in the joined rooms at the same time.
 * @param roomId The string matches the regular expression: `[a-zA-Z0-9_@\-\.]{1,128}`.
 * @return RTSRoom{@link #RTSRoom} instance.
 * @notes <br>
 *       + If the room that you wish to join already exists, you still need to call this API first to create the RTSRoom instance, and then call joinRoom{@link #RTSRoom#joinRoom}.
 *       + In multi-room mode, you need to use different roomId to create multiple rooms.
 */
- (RTSRoom * _Nullable)createRoom:(NSString * _Nonnull)roomId;

#pragma mark - Real-time Messaging Methods
/** 
 * @type api
 * @region Real-time messaging
 * @brief Logs in to the RTS system. <br>
 *        Login is necessary for sending and receiving messages.<br>
 *        To log out, call logout{@link #RTS#logout}.
 * @param token Dynamic key which is required during login for authentication. <br>
 *        This Token is different from that required by calling joinRoom. You can generate a temporary token on the console when testing. When the application goes live, you need to use the key SDK to generate and issue Token on your server side.
 * @param uid User ID, which needs to be unique within one appid.
 * @return API call result: <br>
 *        + `0`: Success<br>
 *        + `-1`: Failure due to invalid parameter<br>
 *        + `-2`: Invalid call. RTC will return this value when you call this API after the user has been logged in to the server. <br>
 *        + `-3`: Failure. There is no created engine instance.
 * @notes You will receive rtsEngine:onLoginResult:errorCode:elapsed:{@link #RTSDelegate#rtsEngine:onLoginResult:errorCode:elapsed:} after calling this API and log in successfully.
 */
- (int)login:(NSString * _Nonnull)token uid:(NSString * _Nonnull)uid;
/** 
 * @type After api
 * @region real-time messaging
 * @brief Logs out of the RTS system. <br>
 *        After calling this API, it is impossible to call messsage-related APIs or receive related callbacks.
 * @return  <br>
 *        + 0: Success.
 *        + < 0 : Fail. See ByteRTCReturnStatus{@link #ByteRTCReturnStatus} for more details
 * @notes After this API is called to log out, local users will receive rtcEngineOnLogout:{@link #RTSDelegate#rtsEngineOnLogout:} notifying the result, and remote users will not receive any notifications.
 */
- (int)logout;
/** 
 * @type api
 * @region Real-time messaging
 * @brief Update the Token <br>
 *        Token used by the user to log in has a certain valid period. When the Token expires, you need to call this method to update the login Token information. <br>
 *        If login:uid:{@link #RTS#login:uid:} is called with an expired token, the login will fail and you will receive rtsEngine:onLoginResult:errorCode:elapsed:{@link #RTSDelegate#rtsEngine:onLoginResult:errorCode:elapsed:} with an error code of `ByteRTCLoginErrorCodeInvalidToken`. You need to reacquire the token and call this method to update the token.
 * @param token Updated token. <br>
 * @return  <br>
 *        + 0: Success.
 *        + < 0 : Fail. See ByteRTCReturnStatus{@link #ByteRTCReturnStatus} for more details
 * @notes  <br>
 *        + If the token is invalid and the login fails, call this method to update the token, and the SDK will automatically log in again. The user does not need to call the login:uid:{@link #RTS#login:uid:} method. <br>
 *        + Token expires, if you have successfully logged in, it will not be affected. An expired Token error will be notified to the user the next time you log in with an expired Token, or when you log in again due to a disconnection due to poor local network conditions.
 */
- (int)updateLoginToken:(NSString * _Nonnull)token;
/** 
 * @type api
 * @region Real-time messaging
 * @brief Set application server parameters   <br>
 *        Client side calls sendServerMessage:{@link #RTS#sendServerMessage:} or sendServerBinaryMessage:{@link #RTS#sendServerBinaryMessage:} Before sending a message to the application server, you must set a valid signature and application server address.
 * @param signature Dynamic signature. The App server may use the signature to verify the source of messages.<br>
 *        You need to define the signature yourself. It can be any non-empty string. It is recommended to encode information such as UID into the signature.<br>
 *        The signature will be sent to the address set through the "url" parameter in the form of a POST request.
 * @param url The address of the application server.
 * @return  <br>
 *        + 0: Success.
 *        + < 0 : Fail. See ByteRTCReturnStatus{@link #ByteRTCReturnStatus} for more details
 * @notes  <br>
 *        + The user must call login:uid:{@link #RTS#login:uid:} before logging in to call this API. <br>
 *        + After calling this API, the SDK will use rtsEngine:onServerParamsSetResult:{@link #RTSDelegate#rtsEngine:onServerParamsSetResult:} to return the corresponding result.
 */
- (int)setServerParams:(NSString * _Nonnull)signature url:(NSString * _Nonnull)url;
/** 
 * @type api
 * @region Real-time messaging
 * @brief Queries the login status of local/remote users.<br>
 *        Before sending a message, you can know whether the peer user is logged in through this API to decide whether to send a message. You can also check your login status through this API.
 * @param peerUserId ID of the user to be queried.
 * @return  <br>
 *        + 0: Success.
 *        + < 0 : Fail. See ByteRTCReturnStatus{@link #ByteRTCReturnStatus} for more details
 * @notes  <br>
 *        + You must call login:uid:{@link #RTS#login:uid:} After logging in, you can call this API. <br>
 *        + After calling this API, the SDK will use rtsEngine:onGetPeerOnlineStatus:status:{@link #RTSDelegate#rtsEngine:onGetPeerOnlineStatus:status:} callback to notify the query result. <br>
 */
- (int)getPeerOnlineStatus:(NSString * _Nonnull)peerUserId;
/** 
 * @type api
 * @region Real-time messaging
 * @brief Sends a text message to a specified user (p2p).
 * @param userId Message receiver's user ID.
 * @param messageStr Message content, not exceeding 64KB.
 * @return  API call result: <br>
 *         + > 0: Sent successfully, return the number of the sent message, increment from 1 <br>
 *         + -1: Sent failed, there is no created engine instance <br>
 *         + -2: Sent failed, userId is empty
 * @notes  <br>
 *        + Before sending a p2p text message, you should call login:uid:{@link #RTS#login:uid:} to complete login. <br>
 *        + After the user calls this API to send a text message, he will receive an rtsEngine:onUserMessageSendResultOutsideRoom:error:{@link #RTSDelegate#rtsEngine:onUserMessageSendResultOutsideRoom:error:} callback to know whether the message was sent successfully. <br>
 *        + If the text message is sent successfully, the user specified by the userId will receive the message through rtsEngine:onUserMessageReceivedOutsideRoom:message:{@link #RTSDelegate#rtsEngine:onUserMessageReceivedOutsideRoom:message:} callback.
 */
- (int64_t)sendMessage:(NSString * _Nonnull)userId message:(NSString * _Nonnull)messageStr config:(int)config;
/** 
 * @type api
 * @region Real-time messaging
 * @brief Sends binary messages to the specified user outside the room (P2P).
 * @param userId Message receiver's user ID.
 * @param messageStr Message content, not exceeding 46KB.
 * @return API call result: <br>
 *         + > 0: Sent successfully, return the number of the sent message, increment from 1 <br>
 *         + -1: Sent failed, there is no created engine instance <br>
 *         + -2: Sent failed, userId is empty
 * @notes  <br>
 *        + Before sending a p2p binary message, you must call login:uid:{@link #RTS#login:uid:} to complete login. <br>
 *        + After the user calls this API to send a binary message, he will receive an rtsEngine:onUserMessageSendResultOutsideRoom:error:{@link #RTSDelegate#rtsEngine:onUserMessageSendResultOutsideRoom:error:} callback to notify whether the message was sent successfully. <br>
 *        + If the binary message is sent successfully, the user specified by the userId will use rtsEngine:onBinaryMessageReceived:message:{@link #RTSDelegate#rtsEngine:onBinaryMessageReceived:message:} callback Receive the message.
 */
- (int64_t)sendBinaryMessage:(NSString * _Nonnull)userId message:(NSData * _Nonnull)messageStr config:(int)config;
/** 
 * @type api
 * @region Real-time messaging
 * @brief Sends a text message to the application server (P2Server).
 * @param messageStr Message content, not exceeding 64KB.
 * @return API call result: <br>
 *         + > 0: Sent successfully, return the number of the sent message, increment from 1 <br>
 *         + -1: Sent failed, there is no created engine instance
 * @notes  <br>
 *        + Before sending a text message to the application server, you must first call login:uid:{@link #RTS#login:uid:} to complete the login, and then call setServerParams:url:{@link #RTS#setServerParams:url:} Set up the application server. <br>
 *        + After calling this API, you will receive an rtsEngine:onServerMessageSendResult:error:message:{@link #RTSDelegate#rtsEngine:onServerMessageSendResult:error:message:} callback to inform the message sender whether the message was sent successfully. <br>
 *        + If the text message is sent successfully, the application server that previously called setServerParams:url:{@link #RTS#setServerParams:url:} will receive the message.
 */
- (int64_t)sendServerMessage:(NSString * _Nonnull)messageStr;
/** 
 * @type api
 * @region Real-time messaging
 * @brief Sends binary messages to the application server (P2Server).
 * @param messageStr Message content, not exceedingå 46KB.
 * @return API call result: <br>
 *         + > 0: Send successfully, return the number of the sent message, increment from 1 <br>
 *         + -1: Send failed, there is no created engine instance
 * @notes  <br>
 *        + Before sending a binary message to the application server, call login:uid:{@link #RTS#login:uid:} to complete the login, then call setServerParams:url:{@link #RTS#setServerParams:url:} Set up the application server. <br>
 *        + After calling this API, you will receive an rtsEngine:onServerMessageSendResult:error:message:{@link #RTSDelegate#rtsEngine:onServerMessageSendResult:error:message:} callback to inform the message sender that the sending succeeded or failed; <br>
 *        + If the binary message is sent successfully, the application server that previously called setServerParams:url:{@link #RTS#setServerParams:url:} will receive the message.
 */
- (int64_t)sendServerBinaryMessage:(NSData *_Nonnull)messageStr;
/** 
 * @type api
 * @brief Enable cellular network assisted communication to improve call quality.
 * @param enhance Enable or not. Off by default.
 * @return Method call result:   <br>
 *        + 0: Success. <br>
 *        + -1: Failure, internal error. <br>
 */
- (int)setCellularEnhancement:(BOOL)enhance;

/** 
 * @type api
 * @region proxy
 * @brief Sets local proxy.
 * @param configurations Local proxy configurations. Refer to ByteRTCLocalProxyInfo{@link #ByteRTCLocalProxyInfo} for details. <br>
 *         You can set both Http tunnel and Socks5 as your local proxies, or only set one of them based on your needs. If you set both Http tunnel and Socks5 as your local proxies, then media traffic and signaling are routed through Socks5 proxy and Http requests through Http tunnel proxy. If you set either Http tunnel or Socks5 as your local proxy, then media traffic, signaling and Http requests are all routed through the proxy you chose. <br>
 *         If you want to remove the existing local proxy configurations, you can call this API with the parameter set to null. 
 * @notes <br>
 *       + You must call this API before joining the room.  <br>
 *       + After calling this API, you will receive rtcEngine:onLocalProxyStateChanged:withProxyState:withProxyError:{@link #RTSDelegate#rtcEngine:onLocalProxyStateChanged:withProxyState:withProxyError:withProxyError:} callback that informs you of the states of local proxy connection. 
 */
- (int)setLocalProxy:(NSArray <ByteRTCLocalProxyInfo *> * _Nonnull)configurations;

@end
